# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["ChatCreateParams"]


class ChatCreateParams(TypedDict, total=False):
    idle_timeout_ms: Annotated[Optional[int], PropertyInfo(alias="idleTimeoutMs")]
    """Idle timeout before session is eligible for snapshot/termination.

    Defaults to 15 minutes.
    """

    model: Optional[str]
    """Optional model override for the OpenCode session"""

    title: str
    """Optional human-readable session title"""

    vault_ids: Annotated[Optional[SequenceNotStr[str]], PropertyInfo(alias="vaultIds")]
    """Restrict the chat session to specific vault IDs"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["SkillUpdateParams"]


class SkillUpdateParams(TypedDict, total=False):
    content: str

    metadata: object

    name: str

    body_slug: Annotated[str, PropertyInfo(alias="slug")]
    """New slug (renames the skill)"""

    summary: Optional[str]

    tags: SequenceNotStr[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from casedev import Casedev, AsyncCasedev
from tests.utils import assert_matches_type
from casedev.types.agent.v1 import (
    ChatCancelResponse,
    ChatCreateResponse,
    ChatDeleteResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestChat:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Casedev) -> None:
        chat = client.agent.v1.chat.create()
        assert_matches_type(ChatCreateResponse, chat, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Casedev) -> None:
        chat = client.agent.v1.chat.create(
            idle_timeout_ms=0,
            model="model",
            title="title",
            vault_ids=["string"],
        )
        assert_matches_type(ChatCreateResponse, chat, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Casedev) -> None:
        response = client.agent.v1.chat.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(ChatCreateResponse, chat, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Casedev) -> None:
        with client.agent.v1.chat.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(ChatCreateResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: Casedev) -> None:
        chat = client.agent.v1.chat.delete(
            "id",
        )
        assert_matches_type(ChatDeleteResponse, chat, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: Casedev) -> None:
        response = client.agent.v1.chat.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(ChatDeleteResponse, chat, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: Casedev) -> None:
        with client.agent.v1.chat.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(ChatDeleteResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v1.chat.with_raw_response.delete(
                "",
            )

    @parametrize
    def test_method_cancel(self, client: Casedev) -> None:
        chat = client.agent.v1.chat.cancel(
            "id",
        )
        assert_matches_type(ChatCancelResponse, chat, path=["response"])

    @parametrize
    def test_raw_response_cancel(self, client: Casedev) -> None:
        response = client.agent.v1.chat.with_raw_response.cancel(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert_matches_type(ChatCancelResponse, chat, path=["response"])

    @parametrize
    def test_streaming_response_cancel(self, client: Casedev) -> None:
        with client.agent.v1.chat.with_streaming_response.cancel(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert_matches_type(ChatCancelResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_cancel(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v1.chat.with_raw_response.cancel(
                "",
            )

    @parametrize
    def test_method_reply_to_question(self, client: Casedev) -> None:
        chat = client.agent.v1.chat.reply_to_question(
            request_id="requestID",
            id="id",
            answers=[["string"]],
        )
        assert chat is None

    @parametrize
    def test_raw_response_reply_to_question(self, client: Casedev) -> None:
        response = client.agent.v1.chat.with_raw_response.reply_to_question(
            request_id="requestID",
            id="id",
            answers=[["string"]],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert chat is None

    @parametrize
    def test_streaming_response_reply_to_question(self, client: Casedev) -> None:
        with client.agent.v1.chat.with_streaming_response.reply_to_question(
            request_id="requestID",
            id="id",
            answers=[["string"]],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert chat is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_reply_to_question(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v1.chat.with_raw_response.reply_to_question(
                request_id="requestID",
                id="",
                answers=[["string"]],
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `request_id` but received ''"):
            client.agent.v1.chat.with_raw_response.reply_to_question(
                request_id="",
                id="id",
                answers=[["string"]],
            )

    @parametrize
    def test_method_respond(self, client: Casedev) -> None:
        chat_stream = client.agent.v1.chat.respond(
            id="id",
        )
        chat_stream.response.close()

    @parametrize
    def test_method_respond_with_all_params(self, client: Casedev) -> None:
        chat_stream = client.agent.v1.chat.respond(
            id="id",
            parts=[
                {
                    "text": "text",
                    "type": "text",
                }
            ],
        )
        chat_stream.response.close()

    @parametrize
    def test_raw_response_respond(self, client: Casedev) -> None:
        response = client.agent.v1.chat.with_raw_response.respond(
            id="id",
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @parametrize
    def test_streaming_response_respond(self, client: Casedev) -> None:
        with client.agent.v1.chat.with_streaming_response.respond(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_respond(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v1.chat.with_raw_response.respond(
                id="",
            )

    @parametrize
    def test_method_send_message(self, client: Casedev) -> None:
        chat = client.agent.v1.chat.send_message(
            id="id",
        )
        assert chat is None

    @parametrize
    def test_method_send_message_with_all_params(self, client: Casedev) -> None:
        chat = client.agent.v1.chat.send_message(
            id="id",
            parts=[
                {
                    "text": "text",
                    "type": "text",
                }
            ],
        )
        assert chat is None

    @parametrize
    def test_raw_response_send_message(self, client: Casedev) -> None:
        response = client.agent.v1.chat.with_raw_response.send_message(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = response.parse()
        assert chat is None

    @parametrize
    def test_streaming_response_send_message(self, client: Casedev) -> None:
        with client.agent.v1.chat.with_streaming_response.send_message(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = response.parse()
            assert chat is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_send_message(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v1.chat.with_raw_response.send_message(
                id="",
            )

    @parametrize
    def test_method_stream(self, client: Casedev) -> None:
        chat_stream = client.agent.v1.chat.stream(
            id="id",
        )
        chat_stream.response.close()

    @parametrize
    def test_method_stream_with_all_params(self, client: Casedev) -> None:
        chat_stream = client.agent.v1.chat.stream(
            id="id",
            last_event_id=0,
        )
        chat_stream.response.close()

    @parametrize
    def test_raw_response_stream(self, client: Casedev) -> None:
        response = client.agent.v1.chat.with_raw_response.stream(
            id="id",
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @parametrize
    def test_streaming_response_stream(self, client: Casedev) -> None:
        with client.agent.v1.chat.with_streaming_response.stream(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_stream(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v1.chat.with_raw_response.stream(
                id="",
            )


class TestAsyncChat:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncCasedev) -> None:
        chat = await async_client.agent.v1.chat.create()
        assert_matches_type(ChatCreateResponse, chat, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncCasedev) -> None:
        chat = await async_client.agent.v1.chat.create(
            idle_timeout_ms=0,
            model="model",
            title="title",
            vault_ids=["string"],
        )
        assert_matches_type(ChatCreateResponse, chat, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.chat.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(ChatCreateResponse, chat, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.chat.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(ChatCreateResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncCasedev) -> None:
        chat = await async_client.agent.v1.chat.delete(
            "id",
        )
        assert_matches_type(ChatDeleteResponse, chat, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.chat.with_raw_response.delete(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(ChatDeleteResponse, chat, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.chat.with_streaming_response.delete(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(ChatDeleteResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v1.chat.with_raw_response.delete(
                "",
            )

    @parametrize
    async def test_method_cancel(self, async_client: AsyncCasedev) -> None:
        chat = await async_client.agent.v1.chat.cancel(
            "id",
        )
        assert_matches_type(ChatCancelResponse, chat, path=["response"])

    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.chat.with_raw_response.cancel(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert_matches_type(ChatCancelResponse, chat, path=["response"])

    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.chat.with_streaming_response.cancel(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert_matches_type(ChatCancelResponse, chat, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v1.chat.with_raw_response.cancel(
                "",
            )

    @parametrize
    async def test_method_reply_to_question(self, async_client: AsyncCasedev) -> None:
        chat = await async_client.agent.v1.chat.reply_to_question(
            request_id="requestID",
            id="id",
            answers=[["string"]],
        )
        assert chat is None

    @parametrize
    async def test_raw_response_reply_to_question(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.chat.with_raw_response.reply_to_question(
            request_id="requestID",
            id="id",
            answers=[["string"]],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert chat is None

    @parametrize
    async def test_streaming_response_reply_to_question(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.chat.with_streaming_response.reply_to_question(
            request_id="requestID",
            id="id",
            answers=[["string"]],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert chat is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_reply_to_question(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v1.chat.with_raw_response.reply_to_question(
                request_id="requestID",
                id="",
                answers=[["string"]],
            )

        with pytest.raises(ValueError, match=r"Expected a non-empty value for `request_id` but received ''"):
            await async_client.agent.v1.chat.with_raw_response.reply_to_question(
                request_id="",
                id="id",
                answers=[["string"]],
            )

    @parametrize
    async def test_method_respond(self, async_client: AsyncCasedev) -> None:
        chat_stream = await async_client.agent.v1.chat.respond(
            id="id",
        )
        await chat_stream.response.aclose()

    @parametrize
    async def test_method_respond_with_all_params(self, async_client: AsyncCasedev) -> None:
        chat_stream = await async_client.agent.v1.chat.respond(
            id="id",
            parts=[
                {
                    "text": "text",
                    "type": "text",
                }
            ],
        )
        await chat_stream.response.aclose()

    @parametrize
    async def test_raw_response_respond(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.chat.with_raw_response.respond(
            id="id",
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = await response.parse()
        await stream.close()

    @parametrize
    async def test_streaming_response_respond(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.chat.with_streaming_response.respond(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_respond(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v1.chat.with_raw_response.respond(
                id="",
            )

    @parametrize
    async def test_method_send_message(self, async_client: AsyncCasedev) -> None:
        chat = await async_client.agent.v1.chat.send_message(
            id="id",
        )
        assert chat is None

    @parametrize
    async def test_method_send_message_with_all_params(self, async_client: AsyncCasedev) -> None:
        chat = await async_client.agent.v1.chat.send_message(
            id="id",
            parts=[
                {
                    "text": "text",
                    "type": "text",
                }
            ],
        )
        assert chat is None

    @parametrize
    async def test_raw_response_send_message(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.chat.with_raw_response.send_message(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        chat = await response.parse()
        assert chat is None

    @parametrize
    async def test_streaming_response_send_message(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.chat.with_streaming_response.send_message(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            chat = await response.parse()
            assert chat is None

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_send_message(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v1.chat.with_raw_response.send_message(
                id="",
            )

    @parametrize
    async def test_method_stream(self, async_client: AsyncCasedev) -> None:
        chat_stream = await async_client.agent.v1.chat.stream(
            id="id",
        )
        await chat_stream.response.aclose()

    @parametrize
    async def test_method_stream_with_all_params(self, async_client: AsyncCasedev) -> None:
        chat_stream = await async_client.agent.v1.chat.stream(
            id="id",
            last_event_id=0,
        )
        await chat_stream.response.aclose()

    @parametrize
    async def test_raw_response_stream(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v1.chat.with_raw_response.stream(
            id="id",
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = await response.parse()
        await stream.close()

    @parametrize
    async def test_streaming_response_stream(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v1.chat.with_streaming_response.stream(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_stream(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v1.chat.with_raw_response.stream(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["InboxSetPolicyParams"]


class InboxSetPolicyParams(TypedDict, total=False):
    allowed_sender_patterns: Annotated[SequenceNotStr[str], PropertyInfo(alias="allowedSenderPatterns")]
    """Exact emails, @domain rules, or \\**"""

    enforce_sender_allowlist: Annotated[bool, PropertyInfo(alias="enforceSenderAllowlist")]

    read_access_rules: Annotated[SequenceNotStr[str], PropertyInfo(alias="readAccessRules")]
    """
    Rules like organization, operator, user:<id>, api_key, api_key:<id>,
    clerk_session, or \\**
    """

    reply_access_rules: Annotated[SequenceNotStr[str], PropertyInfo(alias="replyAccessRules")]
    """
    Rules like organization, operator, user:<id>, api_key, api_key:<id>,
    clerk_session, or \\**
    """

    send_access_rules: Annotated[SequenceNotStr[str], PropertyInfo(alias="sendAccessRules")]
    """Rules like organization, user:<id>, api_key, api_key:<id>, clerk_session, or \\**"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["V1GetCitationsFromURLResponse", "Citations", "CitationsCase", "CitationsRegulation", "CitationsStatute"]


class CitationsCase(BaseModel):
    citation: Optional[str] = None
    """The citation string"""

    count: Optional[int] = None
    """Number of occurrences"""

    type: Optional[str] = None
    """Citation type (usReporter, federalReporter, etc.)"""


class CitationsRegulation(BaseModel):
    citation: Optional[str] = None
    """The citation string"""

    count: Optional[int] = None
    """Number of occurrences"""

    type: Optional[str] = None
    """Citation type (cfr)"""


class CitationsStatute(BaseModel):
    citation: Optional[str] = None
    """The citation string"""

    count: Optional[int] = None
    """Number of occurrences"""

    type: Optional[str] = None
    """Citation type (usc)"""


class Citations(BaseModel):
    cases: Optional[List[CitationsCase]] = None

    regulations: Optional[List[CitationsRegulation]] = None

    statutes: Optional[List[CitationsStatute]] = None


class V1GetCitationsFromURLResponse(BaseModel):
    citations: Optional[Citations] = None

    external_links: Optional[List[str]] = FieldInfo(alias="externalLinks", default=None)
    """External links found in the document"""

    hint: Optional[str] = None
    """Usage guidance"""

    title: Optional[str] = None
    """Document title"""

    total_citations: Optional[int] = FieldInfo(alias="totalCitations", default=None)
    """Total citations found"""

    url: Optional[str] = None
    """Source document URL"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["ProjectListBranchesResponse", "Branch"]


class Branch(BaseModel):
    id: Optional[str] = None
    """Branch ID"""

    created_at: Optional[datetime] = FieldInfo(alias="createdAt", default=None)
    """Branch creation timestamp"""

    is_default: Optional[bool] = FieldInfo(alias="isDefault", default=None)
    """Whether this is the default branch"""

    name: Optional[str] = None
    """Branch name"""

    parent_branch_id: Optional[str] = FieldInfo(alias="parentBranchId", default=None)
    """Parent branch ID (null for default branch)"""

    status: Optional[str] = None
    """Branch status"""

    updated_at: Optional[datetime] = FieldInfo(alias="updatedAt", default=None)
    """Branch last update timestamp"""


class ProjectListBranchesResponse(BaseModel):
    branches: List[Branch]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["V1ProcessResponse"]


class V1ProcessResponse(BaseModel):
    id: Optional[str] = None
    """Unique job identifier"""

    created_at: Optional[datetime] = None
    """Job creation timestamp"""

    document_id: Optional[str] = None
    """Document identifier"""

    engine: Optional[str] = None
    """OCR engine used"""

    estimated_completion: Optional[datetime] = None
    """Estimated completion time"""

    page_count: Optional[int] = None
    """Number of pages detected"""

    status: Optional[Literal["queued", "processing", "completed", "failed"]] = None
    """Current job status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["InstanceTypeListResponse", "InstanceType", "InstanceTypeSpecs"]


class InstanceTypeSpecs(BaseModel):
    memory_gib: Optional[int] = FieldInfo(alias="memoryGib", default=None)
    """RAM in GiB"""

    storage_gib: Optional[int] = FieldInfo(alias="storageGib", default=None)
    """Storage in GiB"""

    vcpus: Optional[int] = None
    """Number of vCPUs"""


class InstanceType(BaseModel):
    description: Optional[str] = None
    """Instance description"""

    gpu: Optional[str] = None
    """GPU model and count"""

    name: Optional[str] = None
    """Instance type identifier"""

    price_per_hour: Optional[str] = FieldInfo(alias="pricePerHour", default=None)
    """Price per hour (e.g. '$1.20')"""

    regions_available: Optional[List[str]] = FieldInfo(alias="regionsAvailable", default=None)
    """Available regions"""

    specs: Optional[InstanceTypeSpecs] = None


class InstanceTypeListResponse(BaseModel):
    count: int
    """Total number of instance types"""

    instance_types: List[InstanceType] = FieldInfo(alias="instanceTypes")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from casedev import Casedev, AsyncCasedev
from tests.utils import assert_matches_type
from casedev.types.agent.v2 import (
    RunExecResponse,
    RunCreateResponse,
    RunGetStatusResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestRun:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create(self, client: Casedev) -> None:
        run = client.agent.v2.run.create(
            agent_id="agentId",
            prompt="prompt",
        )
        assert_matches_type(RunCreateResponse, run, path=["response"])

    @parametrize
    def test_method_create_with_all_params(self, client: Casedev) -> None:
        run = client.agent.v2.run.create(
            agent_id="agentId",
            prompt="prompt",
            callback_url="https://example.com",
            guidance="guidance",
            model="model",
            object_ids=["string"],
        )
        assert_matches_type(RunCreateResponse, run, path=["response"])

    @parametrize
    def test_raw_response_create(self, client: Casedev) -> None:
        response = client.agent.v2.run.with_raw_response.create(
            agent_id="agentId",
            prompt="prompt",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(RunCreateResponse, run, path=["response"])

    @parametrize
    def test_streaming_response_create(self, client: Casedev) -> None:
        with client.agent.v2.run.with_streaming_response.create(
            agent_id="agentId",
            prompt="prompt",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = response.parse()
            assert_matches_type(RunCreateResponse, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_events(self, client: Casedev) -> None:
        run_stream = client.agent.v2.run.events(
            id="id",
        )
        run_stream.response.close()

    @parametrize
    def test_method_events_with_all_params(self, client: Casedev) -> None:
        run_stream = client.agent.v2.run.events(
            id="id",
            last_event_id=0,
        )
        run_stream.response.close()

    @parametrize
    def test_raw_response_events(self, client: Casedev) -> None:
        response = client.agent.v2.run.with_raw_response.events(
            id="id",
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @parametrize
    def test_streaming_response_events(self, client: Casedev) -> None:
        with client.agent.v2.run.with_streaming_response.events(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_events(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v2.run.with_raw_response.events(
                id="",
            )

    @parametrize
    def test_method_exec(self, client: Casedev) -> None:
        run = client.agent.v2.run.exec(
            "id",
        )
        assert_matches_type(RunExecResponse, run, path=["response"])

    @parametrize
    def test_raw_response_exec(self, client: Casedev) -> None:
        response = client.agent.v2.run.with_raw_response.exec(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(RunExecResponse, run, path=["response"])

    @parametrize
    def test_streaming_response_exec(self, client: Casedev) -> None:
        with client.agent.v2.run.with_streaming_response.exec(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = response.parse()
            assert_matches_type(RunExecResponse, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_exec(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v2.run.with_raw_response.exec(
                "",
            )

    @parametrize
    def test_method_get_details(self, client: Casedev) -> None:
        run = client.agent.v2.run.get_details(
            "id",
        )
        assert_matches_type(object, run, path=["response"])

    @parametrize
    def test_raw_response_get_details(self, client: Casedev) -> None:
        response = client.agent.v2.run.with_raw_response.get_details(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(object, run, path=["response"])

    @parametrize
    def test_streaming_response_get_details(self, client: Casedev) -> None:
        with client.agent.v2.run.with_streaming_response.get_details(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = response.parse()
            assert_matches_type(object, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_get_details(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v2.run.with_raw_response.get_details(
                "",
            )

    @parametrize
    def test_method_get_status(self, client: Casedev) -> None:
        run = client.agent.v2.run.get_status(
            "id",
        )
        assert_matches_type(RunGetStatusResponse, run, path=["response"])

    @parametrize
    def test_raw_response_get_status(self, client: Casedev) -> None:
        response = client.agent.v2.run.with_raw_response.get_status(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = response.parse()
        assert_matches_type(RunGetStatusResponse, run, path=["response"])

    @parametrize
    def test_streaming_response_get_status(self, client: Casedev) -> None:
        with client.agent.v2.run.with_streaming_response.get_status(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = response.parse()
            assert_matches_type(RunGetStatusResponse, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_get_status(self, client: Casedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.agent.v2.run.with_raw_response.get_status(
                "",
            )


class TestAsyncRun:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create(self, async_client: AsyncCasedev) -> None:
        run = await async_client.agent.v2.run.create(
            agent_id="agentId",
            prompt="prompt",
        )
        assert_matches_type(RunCreateResponse, run, path=["response"])

    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncCasedev) -> None:
        run = await async_client.agent.v2.run.create(
            agent_id="agentId",
            prompt="prompt",
            callback_url="https://example.com",
            guidance="guidance",
            model="model",
            object_ids=["string"],
        )
        assert_matches_type(RunCreateResponse, run, path=["response"])

    @parametrize
    async def test_raw_response_create(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v2.run.with_raw_response.create(
            agent_id="agentId",
            prompt="prompt",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = await response.parse()
        assert_matches_type(RunCreateResponse, run, path=["response"])

    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v2.run.with_streaming_response.create(
            agent_id="agentId",
            prompt="prompt",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = await response.parse()
            assert_matches_type(RunCreateResponse, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_events(self, async_client: AsyncCasedev) -> None:
        run_stream = await async_client.agent.v2.run.events(
            id="id",
        )
        await run_stream.response.aclose()

    @parametrize
    async def test_method_events_with_all_params(self, async_client: AsyncCasedev) -> None:
        run_stream = await async_client.agent.v2.run.events(
            id="id",
            last_event_id=0,
        )
        await run_stream.response.aclose()

    @parametrize
    async def test_raw_response_events(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v2.run.with_raw_response.events(
            id="id",
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = await response.parse()
        await stream.close()

    @parametrize
    async def test_streaming_response_events(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v2.run.with_streaming_response.events(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_events(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v2.run.with_raw_response.events(
                id="",
            )

    @parametrize
    async def test_method_exec(self, async_client: AsyncCasedev) -> None:
        run = await async_client.agent.v2.run.exec(
            "id",
        )
        assert_matches_type(RunExecResponse, run, path=["response"])

    @parametrize
    async def test_raw_response_exec(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v2.run.with_raw_response.exec(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = await response.parse()
        assert_matches_type(RunExecResponse, run, path=["response"])

    @parametrize
    async def test_streaming_response_exec(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v2.run.with_streaming_response.exec(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = await response.parse()
            assert_matches_type(RunExecResponse, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_exec(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v2.run.with_raw_response.exec(
                "",
            )

    @parametrize
    async def test_method_get_details(self, async_client: AsyncCasedev) -> None:
        run = await async_client.agent.v2.run.get_details(
            "id",
        )
        assert_matches_type(object, run, path=["response"])

    @parametrize
    async def test_raw_response_get_details(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v2.run.with_raw_response.get_details(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = await response.parse()
        assert_matches_type(object, run, path=["response"])

    @parametrize
    async def test_streaming_response_get_details(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v2.run.with_streaming_response.get_details(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = await response.parse()
            assert_matches_type(object, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_get_details(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v2.run.with_raw_response.get_details(
                "",
            )

    @parametrize
    async def test_method_get_status(self, async_client: AsyncCasedev) -> None:
        run = await async_client.agent.v2.run.get_status(
            "id",
        )
        assert_matches_type(RunGetStatusResponse, run, path=["response"])

    @parametrize
    async def test_raw_response_get_status(self, async_client: AsyncCasedev) -> None:
        response = await async_client.agent.v2.run.with_raw_response.get_status(
            "id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        run = await response.parse()
        assert_matches_type(RunGetStatusResponse, run, path=["response"])

    @parametrize
    async def test_streaming_response_get_status(self, async_client: AsyncCasedev) -> None:
        async with async_client.agent.v2.run.with_streaming_response.get_status(
            "id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            run = await response.parse()
            assert_matches_type(RunGetStatusResponse, run, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_get_status(self, async_client: AsyncCasedev) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.agent.v2.run.with_raw_response.get_status(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["ProjectCreateBranchResponse"]


class ProjectCreateBranchResponse(BaseModel):
    id: str
    """Branch ID"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Branch creation timestamp"""

    is_default: bool = FieldInfo(alias="isDefault")
    """Whether this is the default branch (always false for new branches)"""

    name: str
    """Branch name"""

    parent_branch_id: Optional[str] = FieldInfo(alias="parentBranchId", default=None)
    """Parent branch ID"""

    status: str
    """Branch status"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from .templates import (
    TemplatesResource,
    AsyncTemplatesResource,
    TemplatesResourceWithRawResponse,
    AsyncTemplatesResourceWithRawResponse,
    TemplatesResourceWithStreamingResponse,
    AsyncTemplatesResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    BinaryAPIResponse,
    AsyncBinaryAPIResponse,
    StreamedBinaryAPIResponse,
    AsyncStreamedBinaryAPIResponse,
    to_custom_raw_response_wrapper,
    to_custom_streamed_response_wrapper,
    async_to_custom_raw_response_wrapper,
    async_to_custom_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.format import v1_create_document_params

__all__ = ["V1Resource", "AsyncV1Resource"]


class V1Resource(SyncAPIResource):
    """Document formatting and template rendering (MD/JSON to PDF/DOCX)"""

    @cached_property
    def templates(self) -> TemplatesResource:
        """Document formatting and template rendering (MD/JSON to PDF/DOCX)"""
        return TemplatesResource(self._client)

    @cached_property
    def with_raw_response(self) -> V1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return V1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> V1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return V1ResourceWithStreamingResponse(self)

    def create_document(
        self,
        *,
        content: str,
        output_format: Literal["pdf", "docx", "html_preview"],
        input_format: Literal["md", "json", "text"] | Omit = omit,
        options: v1_create_document_params.Options | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BinaryAPIResponse:
        """
        Convert Markdown, JSON, or text content to professionally formatted PDF, DOCX,
        or HTML documents. Supports template components with variable interpolation for
        creating consistent legal documents like contracts, briefs, and reports.

        Args:
          content: The source content to format

          output_format: Desired output format

          input_format: Format of the input content

          options: Optional template composition and styling settings used during document
              generation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/pdf", **(extra_headers or {})}
        return self._post(
            "/format/v1/document",
            body=maybe_transform(
                {
                    "content": content,
                    "output_format": output_format,
                    "input_format": input_format,
                    "options": options,
                },
                v1_create_document_params.V1CreateDocumentParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BinaryAPIResponse,
        )


class AsyncV1Resource(AsyncAPIResource):
    """Document formatting and template rendering (MD/JSON to PDF/DOCX)"""

    @cached_property
    def templates(self) -> AsyncTemplatesResource:
        """Document formatting and template rendering (MD/JSON to PDF/DOCX)"""
        return AsyncTemplatesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncV1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return AsyncV1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncV1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return AsyncV1ResourceWithStreamingResponse(self)

    async def create_document(
        self,
        *,
        content: str,
        output_format: Literal["pdf", "docx", "html_preview"],
        input_format: Literal["md", "json", "text"] | Omit = omit,
        options: v1_create_document_params.Options | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncBinaryAPIResponse:
        """
        Convert Markdown, JSON, or text content to professionally formatted PDF, DOCX,
        or HTML documents. Supports template components with variable interpolation for
        creating consistent legal documents like contracts, briefs, and reports.

        Args:
          content: The source content to format

          output_format: Desired output format

          input_format: Format of the input content

          options: Optional template composition and styling settings used during document
              generation

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/pdf", **(extra_headers or {})}
        return await self._post(
            "/format/v1/document",
            body=await async_maybe_transform(
                {
                    "content": content,
                    "output_format": output_format,
                    "input_format": input_format,
                    "options": options,
                },
                v1_create_document_params.V1CreateDocumentParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AsyncBinaryAPIResponse,
        )


class V1ResourceWithRawResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

        self.create_document = to_custom_raw_response_wrapper(
            v1.create_document,
            BinaryAPIResponse,
        )

    @cached_property
    def templates(self) -> TemplatesResourceWithRawResponse:
        """Document formatting and template rendering (MD/JSON to PDF/DOCX)"""
        return TemplatesResourceWithRawResponse(self._v1.templates)


class AsyncV1ResourceWithRawResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

        self.create_document = async_to_custom_raw_response_wrapper(
            v1.create_document,
            AsyncBinaryAPIResponse,
        )

    @cached_property
    def templates(self) -> AsyncTemplatesResourceWithRawResponse:
        """Document formatting and template rendering (MD/JSON to PDF/DOCX)"""
        return AsyncTemplatesResourceWithRawResponse(self._v1.templates)


class V1ResourceWithStreamingResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

        self.create_document = to_custom_streamed_response_wrapper(
            v1.create_document,
            StreamedBinaryAPIResponse,
        )

    @cached_property
    def templates(self) -> TemplatesResourceWithStreamingResponse:
        """Document formatting and template rendering (MD/JSON to PDF/DOCX)"""
        return TemplatesResourceWithStreamingResponse(self._v1.templates)


class AsyncV1ResourceWithStreamingResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

        self.create_document = async_to_custom_streamed_response_wrapper(
            v1.create_document,
            AsyncStreamedBinaryAPIResponse,
        )

    @cached_property
    def templates(self) -> AsyncTemplatesResourceWithStreamingResponse:
        """Document formatting and template rendering (MD/JSON to PDF/DOCX)"""
        return AsyncTemplatesResourceWithStreamingResponse(self._v1.templates)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["V1ListCourtsParams"]


class V1ListCourtsParams(TypedDict, total=False):
    in_use_only: Annotated[bool, PropertyInfo(alias="inUseOnly")]
    """Only return courts with available docket data"""

    jurisdiction: str
    """Optional jurisdiction code filter (e.g.

    FD for Federal District, F for all Federal, S for State)
    """

    limit: int
    """Maximum number of courts to return"""

    offset: int
    """Number of courts to skip before returning results"""

    query: str
    """Search by court name or slug (e.g. "Northern District", "nysd", "ca9")"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["SkillDeleteResponse"]


class SkillDeleteResponse(BaseModel):
    deleted: Optional[bool] = None

    slug: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["StreamingGetURLResponse", "AudioFormat", "Pricing"]


class AudioFormat(BaseModel):
    channels: Optional[int] = None
    """Number of audio channels"""

    encoding: Optional[str] = None
    """Required audio encoding format"""

    sample_rate: Optional[int] = None
    """Required audio sample rate in Hz"""


class Pricing(BaseModel):
    currency: Optional[str] = None
    """Currency for pricing"""

    per_hour: Optional[float] = None
    """Cost per hour of transcription"""

    per_minute: Optional[float] = None
    """Cost per minute of transcription"""


class StreamingGetURLResponse(BaseModel):
    audio_format: Optional[AudioFormat] = None

    connect_url: Optional[str] = None
    """Complete WebSocket URL with authentication token"""

    pricing: Optional[Pricing] = None

    protocol: Optional[str] = None
    """Connection protocol"""

    url: Optional[str] = None
    """Base WebSocket URL for streaming transcription"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional

import httpx

from .custom import (
    CustomResource,
    AsyncCustomResource,
    CustomResourceWithRawResponse,
    AsyncCustomResourceWithRawResponse,
    CustomResourceWithStreamingResponse,
    AsyncCustomResourceWithStreamingResponse,
)
from ...types import skill_create_params, skill_update_params, skill_resolve_params
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.skill_read_response import SkillReadResponse
from ...types.skill_create_response import SkillCreateResponse
from ...types.skill_delete_response import SkillDeleteResponse
from ...types.skill_update_response import SkillUpdateResponse
from ...types.skill_resolve_response import SkillResolveResponse

__all__ = ["SkillsResource", "AsyncSkillsResource"]


class SkillsResource(SyncAPIResource):
    """Search and read legal AI skills for agents"""

    @cached_property
    def custom(self) -> CustomResource:
        """Search and read legal AI skills for agents"""
        return CustomResource(self._client)

    @cached_property
    def with_raw_response(self) -> SkillsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return SkillsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SkillsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return SkillsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        content: str,
        name: str,
        metadata: object | Omit = omit,
        slug: str | Omit = omit,
        summary: str | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SkillCreateResponse:
        """Create an org-scoped custom skill.

        The skill will be searchable via
        /skills/resolve alongside curated skills.

        Args:
          content: Full skill content in markdown

          name: Skill name

          metadata: Arbitrary metadata (author, license, etc.)

          slug: URL-safe slug. Auto-generated from name if omitted.

          summary: Brief description (1-2 sentences)

          tags: Tags for categorization and search boosting

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/skills",
            body=maybe_transform(
                {
                    "content": content,
                    "name": name,
                    "metadata": metadata,
                    "slug": slug,
                    "summary": summary,
                    "tags": tags,
                },
                skill_create_params.SkillCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SkillCreateResponse,
        )

    def update(
        self,
        path_slug: str,
        *,
        content: str | Omit = omit,
        metadata: object | Omit = omit,
        name: str | Omit = omit,
        body_slug: str | Omit = omit,
        summary: Optional[str] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SkillUpdateResponse:
        """Update an org-scoped custom skill by slug.

        Only provided fields are updated.
        Version is auto-incremented.

        Args:
          body_slug: New slug (renames the skill)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_slug:
            raise ValueError(f"Expected a non-empty value for `path_slug` but received {path_slug!r}")
        return self._put(
            path_template("/skills/{path_slug}", path_slug=path_slug),
            body=maybe_transform(
                {
                    "content": content,
                    "metadata": metadata,
                    "name": name,
                    "body_slug": body_slug,
                    "summary": summary,
                    "tags": tags,
                },
                skill_update_params.SkillUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SkillUpdateResponse,
        )

    def delete(
        self,
        slug: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SkillDeleteResponse:
        """Soft-delete an org-scoped custom skill by slug.

        The skill will no longer appear
        in search results.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return self._delete(
            path_template("/skills/{slug}", slug=slug),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SkillDeleteResponse,
        )

    def read(
        self,
        slug: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SkillReadResponse:
        """Read the full content of a legal skill by its slug.

        Returns markdown content,
        tags, and metadata.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return self._get(
            path_template("/skills/{slug}", slug=slug),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SkillReadResponse,
        )

    def resolve(
        self,
        *,
        q: str,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SkillResolveResponse:
        """
        Search the Legal Skills Store using hybrid search (text + tag + semantic).
        Returns ranked results with relevance scores.

        Args:
          q: Search query string

          limit: Maximum number of results to return (1-20)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/skills/resolve",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "q": q,
                        "limit": limit,
                    },
                    skill_resolve_params.SkillResolveParams,
                ),
            ),
            cast_to=SkillResolveResponse,
        )


class AsyncSkillsResource(AsyncAPIResource):
    """Search and read legal AI skills for agents"""

    @cached_property
    def custom(self) -> AsyncCustomResource:
        """Search and read legal AI skills for agents"""
        return AsyncCustomResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSkillsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSkillsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSkillsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return AsyncSkillsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        content: str,
        name: str,
        metadata: object | Omit = omit,
        slug: str | Omit = omit,
        summary: str | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SkillCreateResponse:
        """Create an org-scoped custom skill.

        The skill will be searchable via
        /skills/resolve alongside curated skills.

        Args:
          content: Full skill content in markdown

          name: Skill name

          metadata: Arbitrary metadata (author, license, etc.)

          slug: URL-safe slug. Auto-generated from name if omitted.

          summary: Brief description (1-2 sentences)

          tags: Tags for categorization and search boosting

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/skills",
            body=await async_maybe_transform(
                {
                    "content": content,
                    "name": name,
                    "metadata": metadata,
                    "slug": slug,
                    "summary": summary,
                    "tags": tags,
                },
                skill_create_params.SkillCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SkillCreateResponse,
        )

    async def update(
        self,
        path_slug: str,
        *,
        content: str | Omit = omit,
        metadata: object | Omit = omit,
        name: str | Omit = omit,
        body_slug: str | Omit = omit,
        summary: Optional[str] | Omit = omit,
        tags: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SkillUpdateResponse:
        """Update an org-scoped custom skill by slug.

        Only provided fields are updated.
        Version is auto-incremented.

        Args:
          body_slug: New slug (renames the skill)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not path_slug:
            raise ValueError(f"Expected a non-empty value for `path_slug` but received {path_slug!r}")
        return await self._put(
            path_template("/skills/{path_slug}", path_slug=path_slug),
            body=await async_maybe_transform(
                {
                    "content": content,
                    "metadata": metadata,
                    "name": name,
                    "body_slug": body_slug,
                    "summary": summary,
                    "tags": tags,
                },
                skill_update_params.SkillUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SkillUpdateResponse,
        )

    async def delete(
        self,
        slug: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SkillDeleteResponse:
        """Soft-delete an org-scoped custom skill by slug.

        The skill will no longer appear
        in search results.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return await self._delete(
            path_template("/skills/{slug}", slug=slug),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SkillDeleteResponse,
        )

    async def read(
        self,
        slug: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SkillReadResponse:
        """Read the full content of a legal skill by its slug.

        Returns markdown content,
        tags, and metadata.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return await self._get(
            path_template("/skills/{slug}", slug=slug),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SkillReadResponse,
        )

    async def resolve(
        self,
        *,
        q: str,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SkillResolveResponse:
        """
        Search the Legal Skills Store using hybrid search (text + tag + semantic).
        Returns ranked results with relevance scores.

        Args:
          q: Search query string

          limit: Maximum number of results to return (1-20)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/skills/resolve",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "q": q,
                        "limit": limit,
                    },
                    skill_resolve_params.SkillResolveParams,
                ),
            ),
            cast_to=SkillResolveResponse,
        )


class SkillsResourceWithRawResponse:
    def __init__(self, skills: SkillsResource) -> None:
        self._skills = skills

        self.create = to_raw_response_wrapper(
            skills.create,
        )
        self.update = to_raw_response_wrapper(
            skills.update,
        )
        self.delete = to_raw_response_wrapper(
            skills.delete,
        )
        self.read = to_raw_response_wrapper(
            skills.read,
        )
        self.resolve = to_raw_response_wrapper(
            skills.resolve,
        )

    @cached_property
    def custom(self) -> CustomResourceWithRawResponse:
        """Search and read legal AI skills for agents"""
        return CustomResourceWithRawResponse(self._skills.custom)


class AsyncSkillsResourceWithRawResponse:
    def __init__(self, skills: AsyncSkillsResource) -> None:
        self._skills = skills

        self.create = async_to_raw_response_wrapper(
            skills.create,
        )
        self.update = async_to_raw_response_wrapper(
            skills.update,
        )
        self.delete = async_to_raw_response_wrapper(
            skills.delete,
        )
        self.read = async_to_raw_response_wrapper(
            skills.read,
        )
        self.resolve = async_to_raw_response_wrapper(
            skills.resolve,
        )

    @cached_property
    def custom(self) -> AsyncCustomResourceWithRawResponse:
        """Search and read legal AI skills for agents"""
        return AsyncCustomResourceWithRawResponse(self._skills.custom)


class SkillsResourceWithStreamingResponse:
    def __init__(self, skills: SkillsResource) -> None:
        self._skills = skills

        self.create = to_streamed_response_wrapper(
            skills.create,
        )
        self.update = to_streamed_response_wrapper(
            skills.update,
        )
        self.delete = to_streamed_response_wrapper(
            skills.delete,
        )
        self.read = to_streamed_response_wrapper(
            skills.read,
        )
        self.resolve = to_streamed_response_wrapper(
            skills.resolve,
        )

    @cached_property
    def custom(self) -> CustomResourceWithStreamingResponse:
        """Search and read legal AI skills for agents"""
        return CustomResourceWithStreamingResponse(self._skills.custom)


class AsyncSkillsResourceWithStreamingResponse:
    def __init__(self, skills: AsyncSkillsResource) -> None:
        self._skills = skills

        self.create = async_to_streamed_response_wrapper(
            skills.create,
        )
        self.update = async_to_streamed_response_wrapper(
            skills.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            skills.delete,
        )
        self.read = async_to_streamed_response_wrapper(
            skills.read,
        )
        self.resolve = async_to_streamed_response_wrapper(
            skills.resolve,
        )

    @cached_property
    def custom(self) -> AsyncCustomResourceWithStreamingResponse:
        """Search and read legal AI skills for agents"""
        return AsyncCustomResourceWithStreamingResponse(self._skills.custom)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.mail.v1 import inbox_create_params, inbox_set_policy_params

__all__ = ["InboxesResource", "AsyncInboxesResource"]


class InboxesResource(SyncAPIResource):
    """
    Create, manage, and execute AI agents with tool access, sandbox environments, and async run workflows
    """

    @cached_property
    def with_raw_response(self) -> InboxesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return InboxesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> InboxesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return InboxesResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        address: str | Omit = omit,
        display_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Create an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            "/mail/v1/inboxes",
            body=maybe_transform(
                {
                    "address": address,
                    "display_name": display_name,
                },
                inbox_create_params.InboxCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def retrieve(
        self,
        inbox_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Get an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            path_template("/mail/v1/inboxes/{inbox_id}", inbox_id=inbox_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """List inboxes owned by the authenticated organization."""
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            "/mail/v1/inboxes",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def delete(
        self,
        inbox_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template("/mail/v1/inboxes/{inbox_id}", inbox_id=inbox_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get_attachment(
        self,
        attachment_id: str,
        *,
        inbox_id: str,
        message_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Get attachment metadata for a message in an inbox owned by the authenticated
        organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        if not message_id:
            raise ValueError(f"Expected a non-empty value for `message_id` but received {message_id!r}")
        if not attachment_id:
            raise ValueError(f"Expected a non-empty value for `attachment_id` but received {attachment_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            path_template(
                "/mail/v1/inboxes/{inbox_id}/messages/{message_id}/attachments/{attachment_id}",
                inbox_id=inbox_id,
                message_id=message_id,
                attachment_id=attachment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get_message(
        self,
        message_id: str,
        *,
        inbox_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Get a message for an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        if not message_id:
            raise ValueError(f"Expected a non-empty value for `message_id` but received {message_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            path_template(
                "/mail/v1/inboxes/{inbox_id}/messages/{message_id}", inbox_id=inbox_id, message_id=message_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def get_policy(
        self,
        inbox_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Get the sender allowlist and send/reply/read access rules for an inbox owned by
        the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            path_template("/mail/v1/inboxes/{inbox_id}/policy", inbox_id=inbox_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list_messages(
        self,
        inbox_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        List messages for an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._get(
            path_template("/mail/v1/inboxes/{inbox_id}/messages", inbox_id=inbox_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def reply(
        self,
        message_id: str,
        *,
        inbox_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Reply to a message in an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        if not message_id:
            raise ValueError(f"Expected a non-empty value for `message_id` but received {message_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            path_template(
                "/mail/v1/inboxes/{inbox_id}/messages/{message_id}/reply", inbox_id=inbox_id, message_id=message_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def send(
        self,
        inbox_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Send a message from an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._post(
            path_template("/mail/v1/inboxes/{inbox_id}/messages/send", inbox_id=inbox_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def set_policy(
        self,
        inbox_id: str,
        *,
        allowed_sender_patterns: SequenceNotStr[str] | Omit = omit,
        enforce_sender_allowlist: bool | Omit = omit,
        read_access_rules: SequenceNotStr[str] | Omit = omit,
        reply_access_rules: SequenceNotStr[str] | Omit = omit,
        send_access_rules: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Set the sender allowlist and send/reply/read access rules for an inbox owned by
        the authenticated organization.

        Args:
          allowed_sender_patterns: Exact emails, @domain rules, or \\**

          read_access_rules: Rules like organization, operator, user:<id>, api_key, api_key:<id>,
              clerk_session, or \\**

          reply_access_rules: Rules like organization, operator, user:<id>, api_key, api_key:<id>,
              clerk_session, or \\**

          send_access_rules: Rules like organization, user:<id>, api_key, api_key:<id>, clerk_session, or \\**

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._put(
            path_template("/mail/v1/inboxes/{inbox_id}/policy", inbox_id=inbox_id),
            body=maybe_transform(
                {
                    "allowed_sender_patterns": allowed_sender_patterns,
                    "enforce_sender_allowlist": enforce_sender_allowlist,
                    "read_access_rules": read_access_rules,
                    "reply_access_rules": reply_access_rules,
                    "send_access_rules": send_access_rules,
                },
                inbox_set_policy_params.InboxSetPolicyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncInboxesResource(AsyncAPIResource):
    """
    Create, manage, and execute AI agents with tool access, sandbox environments, and async run workflows
    """

    @cached_property
    def with_raw_response(self) -> AsyncInboxesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return AsyncInboxesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncInboxesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return AsyncInboxesResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        address: str | Omit = omit,
        display_name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Create an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            "/mail/v1/inboxes",
            body=await async_maybe_transform(
                {
                    "address": address,
                    "display_name": display_name,
                },
                inbox_create_params.InboxCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def retrieve(
        self,
        inbox_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Get an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            path_template("/mail/v1/inboxes/{inbox_id}", inbox_id=inbox_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """List inboxes owned by the authenticated organization."""
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            "/mail/v1/inboxes",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def delete(
        self,
        inbox_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template("/mail/v1/inboxes/{inbox_id}", inbox_id=inbox_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get_attachment(
        self,
        attachment_id: str,
        *,
        inbox_id: str,
        message_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Get attachment metadata for a message in an inbox owned by the authenticated
        organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        if not message_id:
            raise ValueError(f"Expected a non-empty value for `message_id` but received {message_id!r}")
        if not attachment_id:
            raise ValueError(f"Expected a non-empty value for `attachment_id` but received {attachment_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            path_template(
                "/mail/v1/inboxes/{inbox_id}/messages/{message_id}/attachments/{attachment_id}",
                inbox_id=inbox_id,
                message_id=message_id,
                attachment_id=attachment_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get_message(
        self,
        message_id: str,
        *,
        inbox_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Get a message for an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        if not message_id:
            raise ValueError(f"Expected a non-empty value for `message_id` but received {message_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            path_template(
                "/mail/v1/inboxes/{inbox_id}/messages/{message_id}", inbox_id=inbox_id, message_id=message_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def get_policy(
        self,
        inbox_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Get the sender allowlist and send/reply/read access rules for an inbox owned by
        the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            path_template("/mail/v1/inboxes/{inbox_id}/policy", inbox_id=inbox_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def list_messages(
        self,
        inbox_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        List messages for an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._get(
            path_template("/mail/v1/inboxes/{inbox_id}/messages", inbox_id=inbox_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def reply(
        self,
        message_id: str,
        *,
        inbox_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Reply to a message in an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        if not message_id:
            raise ValueError(f"Expected a non-empty value for `message_id` but received {message_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            path_template(
                "/mail/v1/inboxes/{inbox_id}/messages/{message_id}/reply", inbox_id=inbox_id, message_id=message_id
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def send(
        self,
        inbox_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Send a message from an inbox owned by the authenticated organization.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._post(
            path_template("/mail/v1/inboxes/{inbox_id}/messages/send", inbox_id=inbox_id),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def set_policy(
        self,
        inbox_id: str,
        *,
        allowed_sender_patterns: SequenceNotStr[str] | Omit = omit,
        enforce_sender_allowlist: bool | Omit = omit,
        read_access_rules: SequenceNotStr[str] | Omit = omit,
        reply_access_rules: SequenceNotStr[str] | Omit = omit,
        send_access_rules: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Set the sender allowlist and send/reply/read access rules for an inbox owned by
        the authenticated organization.

        Args:
          allowed_sender_patterns: Exact emails, @domain rules, or \\**

          read_access_rules: Rules like organization, operator, user:<id>, api_key, api_key:<id>,
              clerk_session, or \\**

          reply_access_rules: Rules like organization, operator, user:<id>, api_key, api_key:<id>,
              clerk_session, or \\**

          send_access_rules: Rules like organization, user:<id>, api_key, api_key:<id>, clerk_session, or \\**

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not inbox_id:
            raise ValueError(f"Expected a non-empty value for `inbox_id` but received {inbox_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._put(
            path_template("/mail/v1/inboxes/{inbox_id}/policy", inbox_id=inbox_id),
            body=await async_maybe_transform(
                {
                    "allowed_sender_patterns": allowed_sender_patterns,
                    "enforce_sender_allowlist": enforce_sender_allowlist,
                    "read_access_rules": read_access_rules,
                    "reply_access_rules": reply_access_rules,
                    "send_access_rules": send_access_rules,
                },
                inbox_set_policy_params.InboxSetPolicyParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class InboxesResourceWithRawResponse:
    def __init__(self, inboxes: InboxesResource) -> None:
        self._inboxes = inboxes

        self.create = to_raw_response_wrapper(
            inboxes.create,
        )
        self.retrieve = to_raw_response_wrapper(
            inboxes.retrieve,
        )
        self.list = to_raw_response_wrapper(
            inboxes.list,
        )
        self.delete = to_raw_response_wrapper(
            inboxes.delete,
        )
        self.get_attachment = to_raw_response_wrapper(
            inboxes.get_attachment,
        )
        self.get_message = to_raw_response_wrapper(
            inboxes.get_message,
        )
        self.get_policy = to_raw_response_wrapper(
            inboxes.get_policy,
        )
        self.list_messages = to_raw_response_wrapper(
            inboxes.list_messages,
        )
        self.reply = to_raw_response_wrapper(
            inboxes.reply,
        )
        self.send = to_raw_response_wrapper(
            inboxes.send,
        )
        self.set_policy = to_raw_response_wrapper(
            inboxes.set_policy,
        )


class AsyncInboxesResourceWithRawResponse:
    def __init__(self, inboxes: AsyncInboxesResource) -> None:
        self._inboxes = inboxes

        self.create = async_to_raw_response_wrapper(
            inboxes.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            inboxes.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            inboxes.list,
        )
        self.delete = async_to_raw_response_wrapper(
            inboxes.delete,
        )
        self.get_attachment = async_to_raw_response_wrapper(
            inboxes.get_attachment,
        )
        self.get_message = async_to_raw_response_wrapper(
            inboxes.get_message,
        )
        self.get_policy = async_to_raw_response_wrapper(
            inboxes.get_policy,
        )
        self.list_messages = async_to_raw_response_wrapper(
            inboxes.list_messages,
        )
        self.reply = async_to_raw_response_wrapper(
            inboxes.reply,
        )
        self.send = async_to_raw_response_wrapper(
            inboxes.send,
        )
        self.set_policy = async_to_raw_response_wrapper(
            inboxes.set_policy,
        )


class InboxesResourceWithStreamingResponse:
    def __init__(self, inboxes: InboxesResource) -> None:
        self._inboxes = inboxes

        self.create = to_streamed_response_wrapper(
            inboxes.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            inboxes.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            inboxes.list,
        )
        self.delete = to_streamed_response_wrapper(
            inboxes.delete,
        )
        self.get_attachment = to_streamed_response_wrapper(
            inboxes.get_attachment,
        )
        self.get_message = to_streamed_response_wrapper(
            inboxes.get_message,
        )
        self.get_policy = to_streamed_response_wrapper(
            inboxes.get_policy,
        )
        self.list_messages = to_streamed_response_wrapper(
            inboxes.list_messages,
        )
        self.reply = to_streamed_response_wrapper(
            inboxes.reply,
        )
        self.send = to_streamed_response_wrapper(
            inboxes.send,
        )
        self.set_policy = to_streamed_response_wrapper(
            inboxes.set_policy,
        )


class AsyncInboxesResourceWithStreamingResponse:
    def __init__(self, inboxes: AsyncInboxesResource) -> None:
        self._inboxes = inboxes

        self.create = async_to_streamed_response_wrapper(
            inboxes.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            inboxes.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            inboxes.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            inboxes.delete,
        )
        self.get_attachment = async_to_streamed_response_wrapper(
            inboxes.get_attachment,
        )
        self.get_message = async_to_streamed_response_wrapper(
            inboxes.get_message,
        )
        self.get_policy = async_to_streamed_response_wrapper(
            inboxes.get_policy,
        )
        self.list_messages = async_to_streamed_response_wrapper(
            inboxes.list_messages,
        )
        self.reply = async_to_streamed_response_wrapper(
            inboxes.reply,
        )
        self.send = async_to_streamed_response_wrapper(
            inboxes.send,
        )
        self.set_policy = async_to_streamed_response_wrapper(
            inboxes.set_policy,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ObjectListResponse", "Object"]


class Object(BaseModel):
    id: str
    """Unique object identifier"""

    content_type: str = FieldInfo(alias="contentType")
    """MIME type of the document"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Document upload timestamp"""

    filename: str
    """Original filename of the uploaded document"""

    ingestion_status: str = FieldInfo(alias="ingestionStatus")
    """Processing status of the document"""

    chunk_count: Optional[float] = FieldInfo(alias="chunkCount", default=None)
    """Number of text chunks created for vectorization"""

    ingestion_completed_at: Optional[datetime] = FieldInfo(alias="ingestionCompletedAt", default=None)
    """Processing completion timestamp"""

    ingestion_error: Optional[str] = FieldInfo(alias="ingestionError", default=None)
    """Failure reason when ingestion status is a failed state"""

    ingestion_started_at: Optional[datetime] = FieldInfo(alias="ingestionStartedAt", default=None)
    """When ingestion processing began"""

    ingestion_workflow_id: Optional[str] = FieldInfo(alias="ingestionWorkflowId", default=None)
    """Durable workflow run ID for the active or last ingestion attempt"""

    metadata: Optional[object] = None
    """Custom metadata associated with the document"""

    page_count: Optional[float] = FieldInfo(alias="pageCount", default=None)
    """Number of pages in the document"""

    path: Optional[str] = None
    """Optional folder path for hierarchy preservation from source systems"""

    size_bytes: Optional[float] = FieldInfo(alias="sizeBytes", default=None)
    """File size in bytes"""

    tags: Optional[List[str]] = None
    """Custom tags associated with the document"""

    text_length: Optional[float] = FieldInfo(alias="textLength", default=None)
    """Total character count of extracted text"""

    vector_count: Optional[float] = FieldInfo(alias="vectorCount", default=None)
    """Number of vectors generated for semantic search"""


class ObjectListResponse(BaseModel):
    count: float
    """Total number of objects in the vault"""

    objects: List[Object]

    vault_id: str = FieldInfo(alias="vaultId")
    """The ID of the vault"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.search import (
    v1_answer_params,
    v1_search_params,
    v1_similar_params,
    v1_contents_params,
    v1_research_params,
    v1_retrieve_research_params,
)
from ...types.search.v1_answer_response import V1AnswerResponse
from ...types.search.v1_search_response import V1SearchResponse
from ...types.search.v1_similar_response import V1SimilarResponse
from ...types.search.v1_contents_response import V1ContentsResponse
from ...types.search.v1_research_response import V1ResearchResponse
from ...types.search.v1_retrieve_research_response import V1RetrieveResearchResponse

__all__ = ["V1Resource", "AsyncV1Resource"]


class V1Resource(SyncAPIResource):
    """Web search, AI answers, and deep research"""

    @cached_property
    def with_raw_response(self) -> V1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return V1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> V1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return V1ResourceWithStreamingResponse(self)

    def answer(
        self,
        *,
        query: str,
        exclude_domains: SequenceNotStr[str] | Omit = omit,
        include_domains: SequenceNotStr[str] | Omit = omit,
        max_tokens: int | Omit = omit,
        model: str | Omit = omit,
        num_results: int | Omit = omit,
        search_type: Literal["auto", "web", "news", "academic"] | Omit = omit,
        stream: bool | Omit = omit,
        temperature: float | Omit = omit,
        text: bool | Omit = omit,
        use_custom_llm: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1AnswerResponse:
        """Generate comprehensive answers to questions using web search results.

        Supports
        two modes: native provider answers or custom LLM-powered answers using
        Case.dev's AI gateway. Perfect for legal research, fact-checking, and gathering
        supporting evidence for cases.

        Args:
          query: The question or topic to research and answer

          exclude_domains: Exclude these domains from search

          include_domains: Only search within these domains

          max_tokens: Maximum tokens for LLM response

          model: LLM model to use when useCustomLLM is true

          num_results: Number of search results to consider

          search_type: Type of search to perform

          stream: Stream the response (only for native provider answers)

          temperature: LLM temperature for answer generation

          text: Include text content in response

          use_custom_llm: Use Case.dev LLM for answer generation instead of provider's native answer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/search/v1/answer",
            body=maybe_transform(
                {
                    "query": query,
                    "exclude_domains": exclude_domains,
                    "include_domains": include_domains,
                    "max_tokens": max_tokens,
                    "model": model,
                    "num_results": num_results,
                    "search_type": search_type,
                    "stream": stream,
                    "temperature": temperature,
                    "text": text,
                    "use_custom_llm": use_custom_llm,
                },
                v1_answer_params.V1AnswerParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1AnswerResponse,
        )

    def contents(
        self,
        *,
        urls: SequenceNotStr[str],
        context: str | Omit = omit,
        extras: object | Omit = omit,
        highlights: bool | Omit = omit,
        livecrawl: bool | Omit = omit,
        livecrawl_timeout: int | Omit = omit,
        subpages: bool | Omit = omit,
        subpage_target: int | Omit = omit,
        summary: bool | Omit = omit,
        text: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1ContentsResponse:
        """Scrapes and extracts text content from web pages, PDFs, and documents.

        Useful
        for legal research, evidence collection, and document analysis. Supports live
        crawling, subpage extraction, and content summarization.

        Args:
          urls: Array of URLs to scrape and extract content from

          context: Context to guide content extraction and summarization

          extras: Additional extraction options

          highlights: Whether to include content highlights

          livecrawl: Whether to perform live crawling for dynamic content

          livecrawl_timeout: Timeout in seconds for live crawling

          subpages: Whether to extract content from linked subpages

          subpage_target: Maximum number of subpages to crawl

          summary: Whether to generate content summaries

          text: Whether to extract text content

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/search/v1/contents",
            body=maybe_transform(
                {
                    "urls": urls,
                    "context": context,
                    "extras": extras,
                    "highlights": highlights,
                    "livecrawl": livecrawl,
                    "livecrawl_timeout": livecrawl_timeout,
                    "subpages": subpages,
                    "subpage_target": subpage_target,
                    "summary": summary,
                    "text": text,
                },
                v1_contents_params.V1ContentsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1ContentsResponse,
        )

    def research(
        self,
        *,
        instructions: str,
        model: Literal["fast", "normal", "pro"] | Omit = omit,
        output_schema: object | Omit = omit,
        query: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1ResearchResponse:
        """
        Performs deep research by conducting multi-step analysis, gathering information
        from multiple sources, and providing comprehensive insights. Ideal for legal
        research, case analysis, and due diligence investigations.

        Args:
          instructions: Research instructions or query

          model: Research quality level - fast (quick), normal (balanced), pro (comprehensive)

          output_schema: Optional JSON schema to structure the research output

          query: Alias for instructions (for convenience)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/search/v1/research",
            body=maybe_transform(
                {
                    "instructions": instructions,
                    "model": model,
                    "output_schema": output_schema,
                    "query": query,
                },
                v1_research_params.V1ResearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1ResearchResponse,
        )

    def retrieve_research(
        self,
        id: str,
        *,
        events: str | Omit = omit,
        stream: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1RetrieveResearchResponse:
        """Retrieve the status and results of a deep research task by ID.

        Supports both
        standard JSON responses and streaming for real-time updates as the research
        progresses. Research tasks analyze topics comprehensively using web search and
        AI synthesis.

        Args:
          events: Filter specific event types for streaming

          stream: Enable streaming for real-time updates

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            path_template("/search/v1/research/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "events": events,
                        "stream": stream,
                    },
                    v1_retrieve_research_params.V1RetrieveResearchParams,
                ),
            ),
            cast_to=V1RetrieveResearchResponse,
        )

    def search(
        self,
        *,
        query: str,
        additional_queries: SequenceNotStr[str] | Omit = omit,
        category: str | Omit = omit,
        contents: str | Omit = omit,
        end_crawl_date: Union[str, date] | Omit = omit,
        end_published_date: Union[str, date] | Omit = omit,
        exclude_domains: SequenceNotStr[str] | Omit = omit,
        include_domains: SequenceNotStr[str] | Omit = omit,
        include_text: bool | Omit = omit,
        num_results: int | Omit = omit,
        start_crawl_date: Union[str, date] | Omit = omit,
        start_published_date: Union[str, date] | Omit = omit,
        type: Literal["auto", "search", "news"] | Omit = omit,
        user_location: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1SearchResponse:
        """
        Executes intelligent web search queries with advanced filtering and
        customization options. Ideal for legal research, case law discovery, and
        gathering supporting documentation for litigation or compliance matters.

        Args:
          query: Primary search query

          additional_queries: Additional related search queries to enhance results

          category: Category filter for search results

          contents: Specific content type to search for

          end_crawl_date: End date for crawl date filtering

          end_published_date: End date for published date filtering

          exclude_domains: Domains to exclude from search results

          include_domains: Domains to include in search results

          include_text: Whether to include full text content in results

          num_results: Number of search results to return

          start_crawl_date: Start date for crawl date filtering

          start_published_date: Start date for published date filtering

          type: Type of search to perform

          user_location: Geographic location for localized results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/search/v1/search",
            body=maybe_transform(
                {
                    "query": query,
                    "additional_queries": additional_queries,
                    "category": category,
                    "contents": contents,
                    "end_crawl_date": end_crawl_date,
                    "end_published_date": end_published_date,
                    "exclude_domains": exclude_domains,
                    "include_domains": include_domains,
                    "include_text": include_text,
                    "num_results": num_results,
                    "start_crawl_date": start_crawl_date,
                    "start_published_date": start_published_date,
                    "type": type,
                    "user_location": user_location,
                },
                v1_search_params.V1SearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1SearchResponse,
        )

    def similar(
        self,
        *,
        url: str,
        contents: str | Omit = omit,
        end_crawl_date: Union[str, date] | Omit = omit,
        end_published_date: Union[str, date] | Omit = omit,
        exclude_domains: SequenceNotStr[str] | Omit = omit,
        include_domains: SequenceNotStr[str] | Omit = omit,
        include_text: bool | Omit = omit,
        num_results: int | Omit = omit,
        start_crawl_date: Union[str, date] | Omit = omit,
        start_published_date: Union[str, date] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1SimilarResponse:
        """Find web pages and documents similar to a given URL.

        Useful for legal research
        to discover related case law, statutes, or legal commentary that shares similar
        themes or content structure.

        Args:
          url: The URL to find similar content for

          contents: Additional content to consider for similarity matching

          end_crawl_date: Only include pages crawled before this date

          end_published_date: Only include pages published before this date

          exclude_domains: Exclude results from these domains

          include_domains: Only search within these domains

          include_text: Whether to include extracted text content in results

          num_results: Number of similar results to return

          start_crawl_date: Only include pages crawled after this date

          start_published_date: Only include pages published after this date

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/search/v1/similar",
            body=maybe_transform(
                {
                    "url": url,
                    "contents": contents,
                    "end_crawl_date": end_crawl_date,
                    "end_published_date": end_published_date,
                    "exclude_domains": exclude_domains,
                    "include_domains": include_domains,
                    "include_text": include_text,
                    "num_results": num_results,
                    "start_crawl_date": start_crawl_date,
                    "start_published_date": start_published_date,
                },
                v1_similar_params.V1SimilarParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1SimilarResponse,
        )


class AsyncV1Resource(AsyncAPIResource):
    """Web search, AI answers, and deep research"""

    @cached_property
    def with_raw_response(self) -> AsyncV1ResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/CaseMark/casedev-python#accessing-raw-response-data-eg-headers
        """
        return AsyncV1ResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncV1ResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/CaseMark/casedev-python#with_streaming_response
        """
        return AsyncV1ResourceWithStreamingResponse(self)

    async def answer(
        self,
        *,
        query: str,
        exclude_domains: SequenceNotStr[str] | Omit = omit,
        include_domains: SequenceNotStr[str] | Omit = omit,
        max_tokens: int | Omit = omit,
        model: str | Omit = omit,
        num_results: int | Omit = omit,
        search_type: Literal["auto", "web", "news", "academic"] | Omit = omit,
        stream: bool | Omit = omit,
        temperature: float | Omit = omit,
        text: bool | Omit = omit,
        use_custom_llm: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1AnswerResponse:
        """Generate comprehensive answers to questions using web search results.

        Supports
        two modes: native provider answers or custom LLM-powered answers using
        Case.dev's AI gateway. Perfect for legal research, fact-checking, and gathering
        supporting evidence for cases.

        Args:
          query: The question or topic to research and answer

          exclude_domains: Exclude these domains from search

          include_domains: Only search within these domains

          max_tokens: Maximum tokens for LLM response

          model: LLM model to use when useCustomLLM is true

          num_results: Number of search results to consider

          search_type: Type of search to perform

          stream: Stream the response (only for native provider answers)

          temperature: LLM temperature for answer generation

          text: Include text content in response

          use_custom_llm: Use Case.dev LLM for answer generation instead of provider's native answer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/search/v1/answer",
            body=await async_maybe_transform(
                {
                    "query": query,
                    "exclude_domains": exclude_domains,
                    "include_domains": include_domains,
                    "max_tokens": max_tokens,
                    "model": model,
                    "num_results": num_results,
                    "search_type": search_type,
                    "stream": stream,
                    "temperature": temperature,
                    "text": text,
                    "use_custom_llm": use_custom_llm,
                },
                v1_answer_params.V1AnswerParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1AnswerResponse,
        )

    async def contents(
        self,
        *,
        urls: SequenceNotStr[str],
        context: str | Omit = omit,
        extras: object | Omit = omit,
        highlights: bool | Omit = omit,
        livecrawl: bool | Omit = omit,
        livecrawl_timeout: int | Omit = omit,
        subpages: bool | Omit = omit,
        subpage_target: int | Omit = omit,
        summary: bool | Omit = omit,
        text: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1ContentsResponse:
        """Scrapes and extracts text content from web pages, PDFs, and documents.

        Useful
        for legal research, evidence collection, and document analysis. Supports live
        crawling, subpage extraction, and content summarization.

        Args:
          urls: Array of URLs to scrape and extract content from

          context: Context to guide content extraction and summarization

          extras: Additional extraction options

          highlights: Whether to include content highlights

          livecrawl: Whether to perform live crawling for dynamic content

          livecrawl_timeout: Timeout in seconds for live crawling

          subpages: Whether to extract content from linked subpages

          subpage_target: Maximum number of subpages to crawl

          summary: Whether to generate content summaries

          text: Whether to extract text content

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/search/v1/contents",
            body=await async_maybe_transform(
                {
                    "urls": urls,
                    "context": context,
                    "extras": extras,
                    "highlights": highlights,
                    "livecrawl": livecrawl,
                    "livecrawl_timeout": livecrawl_timeout,
                    "subpages": subpages,
                    "subpage_target": subpage_target,
                    "summary": summary,
                    "text": text,
                },
                v1_contents_params.V1ContentsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1ContentsResponse,
        )

    async def research(
        self,
        *,
        instructions: str,
        model: Literal["fast", "normal", "pro"] | Omit = omit,
        output_schema: object | Omit = omit,
        query: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1ResearchResponse:
        """
        Performs deep research by conducting multi-step analysis, gathering information
        from multiple sources, and providing comprehensive insights. Ideal for legal
        research, case analysis, and due diligence investigations.

        Args:
          instructions: Research instructions or query

          model: Research quality level - fast (quick), normal (balanced), pro (comprehensive)

          output_schema: Optional JSON schema to structure the research output

          query: Alias for instructions (for convenience)

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/search/v1/research",
            body=await async_maybe_transform(
                {
                    "instructions": instructions,
                    "model": model,
                    "output_schema": output_schema,
                    "query": query,
                },
                v1_research_params.V1ResearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1ResearchResponse,
        )

    async def retrieve_research(
        self,
        id: str,
        *,
        events: str | Omit = omit,
        stream: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1RetrieveResearchResponse:
        """Retrieve the status and results of a deep research task by ID.

        Supports both
        standard JSON responses and streaming for real-time updates as the research
        progresses. Research tasks analyze topics comprehensively using web search and
        AI synthesis.

        Args:
          events: Filter specific event types for streaming

          stream: Enable streaming for real-time updates

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            path_template("/search/v1/research/{id}", id=id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "events": events,
                        "stream": stream,
                    },
                    v1_retrieve_research_params.V1RetrieveResearchParams,
                ),
            ),
            cast_to=V1RetrieveResearchResponse,
        )

    async def search(
        self,
        *,
        query: str,
        additional_queries: SequenceNotStr[str] | Omit = omit,
        category: str | Omit = omit,
        contents: str | Omit = omit,
        end_crawl_date: Union[str, date] | Omit = omit,
        end_published_date: Union[str, date] | Omit = omit,
        exclude_domains: SequenceNotStr[str] | Omit = omit,
        include_domains: SequenceNotStr[str] | Omit = omit,
        include_text: bool | Omit = omit,
        num_results: int | Omit = omit,
        start_crawl_date: Union[str, date] | Omit = omit,
        start_published_date: Union[str, date] | Omit = omit,
        type: Literal["auto", "search", "news"] | Omit = omit,
        user_location: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1SearchResponse:
        """
        Executes intelligent web search queries with advanced filtering and
        customization options. Ideal for legal research, case law discovery, and
        gathering supporting documentation for litigation or compliance matters.

        Args:
          query: Primary search query

          additional_queries: Additional related search queries to enhance results

          category: Category filter for search results

          contents: Specific content type to search for

          end_crawl_date: End date for crawl date filtering

          end_published_date: End date for published date filtering

          exclude_domains: Domains to exclude from search results

          include_domains: Domains to include in search results

          include_text: Whether to include full text content in results

          num_results: Number of search results to return

          start_crawl_date: Start date for crawl date filtering

          start_published_date: Start date for published date filtering

          type: Type of search to perform

          user_location: Geographic location for localized results

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/search/v1/search",
            body=await async_maybe_transform(
                {
                    "query": query,
                    "additional_queries": additional_queries,
                    "category": category,
                    "contents": contents,
                    "end_crawl_date": end_crawl_date,
                    "end_published_date": end_published_date,
                    "exclude_domains": exclude_domains,
                    "include_domains": include_domains,
                    "include_text": include_text,
                    "num_results": num_results,
                    "start_crawl_date": start_crawl_date,
                    "start_published_date": start_published_date,
                    "type": type,
                    "user_location": user_location,
                },
                v1_search_params.V1SearchParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1SearchResponse,
        )

    async def similar(
        self,
        *,
        url: str,
        contents: str | Omit = omit,
        end_crawl_date: Union[str, date] | Omit = omit,
        end_published_date: Union[str, date] | Omit = omit,
        exclude_domains: SequenceNotStr[str] | Omit = omit,
        include_domains: SequenceNotStr[str] | Omit = omit,
        include_text: bool | Omit = omit,
        num_results: int | Omit = omit,
        start_crawl_date: Union[str, date] | Omit = omit,
        start_published_date: Union[str, date] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> V1SimilarResponse:
        """Find web pages and documents similar to a given URL.

        Useful for legal research
        to discover related case law, statutes, or legal commentary that shares similar
        themes or content structure.

        Args:
          url: The URL to find similar content for

          contents: Additional content to consider for similarity matching

          end_crawl_date: Only include pages crawled before this date

          end_published_date: Only include pages published before this date

          exclude_domains: Exclude results from these domains

          include_domains: Only search within these domains

          include_text: Whether to include extracted text content in results

          num_results: Number of similar results to return

          start_crawl_date: Only include pages crawled after this date

          start_published_date: Only include pages published after this date

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/search/v1/similar",
            body=await async_maybe_transform(
                {
                    "url": url,
                    "contents": contents,
                    "end_crawl_date": end_crawl_date,
                    "end_published_date": end_published_date,
                    "exclude_domains": exclude_domains,
                    "include_domains": include_domains,
                    "include_text": include_text,
                    "num_results": num_results,
                    "start_crawl_date": start_crawl_date,
                    "start_published_date": start_published_date,
                },
                v1_similar_params.V1SimilarParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=V1SimilarResponse,
        )


class V1ResourceWithRawResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

        self.answer = to_raw_response_wrapper(
            v1.answer,
        )
        self.contents = to_raw_response_wrapper(
            v1.contents,
        )
        self.research = to_raw_response_wrapper(
            v1.research,
        )
        self.retrieve_research = to_raw_response_wrapper(
            v1.retrieve_research,
        )
        self.search = to_raw_response_wrapper(
            v1.search,
        )
        self.similar = to_raw_response_wrapper(
            v1.similar,
        )


class AsyncV1ResourceWithRawResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

        self.answer = async_to_raw_response_wrapper(
            v1.answer,
        )
        self.contents = async_to_raw_response_wrapper(
            v1.contents,
        )
        self.research = async_to_raw_response_wrapper(
            v1.research,
        )
        self.retrieve_research = async_to_raw_response_wrapper(
            v1.retrieve_research,
        )
        self.search = async_to_raw_response_wrapper(
            v1.search,
        )
        self.similar = async_to_raw_response_wrapper(
            v1.similar,
        )


class V1ResourceWithStreamingResponse:
    def __init__(self, v1: V1Resource) -> None:
        self._v1 = v1

        self.answer = to_streamed_response_wrapper(
            v1.answer,
        )
        self.contents = to_streamed_response_wrapper(
            v1.contents,
        )
        self.research = to_streamed_response_wrapper(
            v1.research,
        )
        self.retrieve_research = to_streamed_response_wrapper(
            v1.retrieve_research,
        )
        self.search = to_streamed_response_wrapper(
            v1.search,
        )
        self.similar = to_streamed_response_wrapper(
            v1.similar,
        )


class AsyncV1ResourceWithStreamingResponse:
    def __init__(self, v1: AsyncV1Resource) -> None:
        self._v1 = v1

        self.answer = async_to_streamed_response_wrapper(
            v1.answer,
        )
        self.contents = async_to_streamed_response_wrapper(
            v1.contents,
        )
        self.research = async_to_streamed_response_wrapper(
            v1.research,
        )
        self.retrieve_research = async_to_streamed_response_wrapper(
            v1.retrieve_research,
        )
        self.search = async_to_streamed_response_wrapper(
            v1.search,
        )
        self.similar = async_to_streamed_response_wrapper(
            v1.similar,
        )
